@@ -17,4 +17,10 @@ ATTRS = {'collection': 'http://pyff-project.org/collection',
          'entity-category': 'http://macedir.org/entity-category',
          'role': 'http://pyff-project.org/role'}
 
-DIGESTS = ['sha1', 'md5', 'null']
\ No newline at end of file
+DIGESTS = ['sha1', 'md5', 'null']
+
+EVENT_DROP_ENTITY = 'event.drop.entity'
+EVENT_RETRY_URL = 'event.retry.url'
+EVENT_IMPORTED_METADATA = 'event.imported.metadata'
+EVENT_IMPORT_FAIL = 'event.import.failed'
+EVENT_REPOSITORY_LIVE = 'event.repository.live'
\ No newline at end of file
