@@ -7,7 +7,7 @@ from StringIO import StringIO
 from datetime import datetime
 import hashlib
 import urllib
-from UserDict import DictMixin
+from UserDict import DictMixin, UserDict
 from lxml import etree
 from lxml.builder import ElementMaker
 from lxml.etree import DocumentInvalid
@@ -19,7 +19,7 @@ import pyff.index
 from pyff.logs import log
 from pyff.utils import schema, URLFetch, filter_lang, root, duration2timedelta, template
 import xmlsec
-from pyff.constants import NS, NF_URI, DIGESTS
+from pyff.constants import NS, NF_URI, DIGESTS, EVENT_DROP_ENTITY, EVENT_IMPORTED_METADATA, EVENT_IMPORT_FAIL
 import traceback
 import threading
 from Queue import Queue
@@ -38,15 +38,40 @@ def _is_self_signed_err(ebuf):
 etree.set_default_parser(etree.XMLParser(resolve_entities=False))
 
 
-def _e(error_log):
-    return "\n".join(filter(lambda x: ":WARNING:" not in x, ["%s" % e for e in error_log]))
+def _e(error_log, m=None):
+    def _f(x):
+        if ":WARNING:" in x:
+            return False
+        if m is not None and not m in x:
+            return False
+        return True
+
+    return "\n".join(filter(_f, ["%s" % e for e in error_log]))
 
 
 class MetadataException(Exception):
     pass
 
 
-class MDRepository(DictMixin):
+class Event(UserDict):
+    pass
+
+
+class Observable(object):
+    def __init__(self):
+        self.callbacks = []
+
+    def subscribe(self, callback):
+        self.callbacks.append(callback)
+
+    def fire(self, **attrs):
+        e = Event(attrs)
+        e['time'] = datetime.now()
+        for fn in self.callbacks:
+            fn(e)
+
+
+class MDRepository(DictMixin, Observable):
     """A class representing a set of SAML Metadata. Instances present as dict-like objects where
     the keys are URIs and values are EntitiesDescriptor elements containing sets of metadata.
     """
@@ -60,6 +85,8 @@ class MDRepository(DictMixin):
         self.default_cache_duration = "PT10M"
         self.retry_limit = 5
 
+        super(MDRepository, self).__init__()
+
     def is_idp(self, entity):
         """Returns True if the supplied EntityDescriptor has an IDPSSODescriptor Role
 
@@ -252,12 +279,12 @@ The dict in the list contains three items:
             a.append(velt)
             #log.debug(etree.tostring(a))
 
-    def fetch_metadata(self, resources, qsize=5, timeout=60, stats=None, xrd=None):
+    def fetch_metadata(self, resources, qsize=5, timeout=120, stats=None, xrd=None):
         """Fetch a series of metadata URLs and optionally verify signatures.
 
 :param resources: A list of triples (url,cert-or-fingerprint,id)
 :param qsize: The number of parallell downloads to run
-:param timeout: The number of seconds to wait (60 by default) for each download
+:param timeout: The number of seconds to wait (120 by default) for each download
 :param stats: A dictionary used for storing statistics. Useful for cherrypy cpstats
 
 The list of triples is processed by first downloading the URL. If a cert-or-fingerprint
@@ -278,7 +305,7 @@ and verified.
         def producer(q, resources, cache=self.metadata_cache_enabled):
             print resources
             for url, verify, id, tries in resources:
-                log.debug("Starting fetcher for %s" % url)
+                log.debug("starting fetcher for '%s'" % url)
                 thread = URLFetch(url, verify, id, enable_cache=cache, tries=tries)
                 thread.start()
                 q.put(thread, True)
@@ -293,12 +320,12 @@ and verified.
             while nfinished < njobs:
                 info = None
                 try:
-                    log.debug("Waiting for next thread to finish...")
+                    log.debug("waiting for next thread to finish...")
                     thread = q.get(True)
                     thread.join(timeout)
 
                     if thread.isAlive():
-                        raise MetadataException("Thread timeout occured")
+                        raise MetadataException("thread timeout fetching '%s'" % thread.url)
 
                     info = {
                         'Time Spent': thread.time()
@@ -310,7 +337,7 @@ and verified.
                         if thread.result is not None:
                             info['Bytes'] = len(thread.result)
                         else:
-                            raise MetadataException("Empty response")
+                            raise MetadataException("empty response fetching '%s'" % thread.url)
                         info['Cached'] = thread.cached
                         info['Date'] = str(thread.date)
                         info['Last-Modified'] = str(thread.last_modified)
@@ -323,7 +350,8 @@ and verified.
 
                     t = self.parse_metadata(StringIO(xml), key=thread.verify, base_url=thread.url)
                     if t is None:
-                        raise MetadataException("No valid metadata found at %s" % thread.url)
+                        self.fire(type=EVENT_IMPORT_FAIL, url=thread.url)
+                        raise MetadataException("no valid metadata found at '%s'" % thread.url)
 
                     relt = root(t)
                     if relt.tag in ('{%s}XRD' % NS['xrd'], '{%s}XRDS' % NS['xrd']):
@@ -348,13 +376,13 @@ and verified.
 
                         if thread.cached:
                             if thread.last_modified + offset < datetime.now() - duration2timedelta(self.min_cache_ttl):
-                                raise MetadataException("Cached metadata expired")
+                                raise MetadataException("cached metadata expired")
                             else:
-                                log.debug("Found cached metadata (last-modified: %s)" % thread.last_modified)
+                                log.debug("found cached metadata for '%s' (last-modified: %s)" % (thread.url, thread.last_modified))
                                 ne = self.import_metadata(t, url=thread.id)
                                 info['Number of Entities'] = ne
                         else:
-                            log.debug("got fresh metadata (date: %s)" % thread.date)
+                            log.debug("got fresh metadata for '%s' (date: %s)" % (thread.url, thread.date))
                             ne = self.import_metadata(t, url=thread.id)
                             info['Number of Entities'] = ne
                         info['Cache Expiration Time'] = str(thread.last_modified + offset)
@@ -364,16 +392,17 @@ and verified.
                             cert = certs.values()[0].strip()
                         resolved.add((thread.url, cert))
                     else:
-                        raise MetadataException("Unknown metadata type (%s)" % relt.tag)
+                        raise MetadataException("unknown metadata type for '%s' (%s)" % (thread.url, relt.tag))
                 except Exception, ex:
-                    traceback.print_exc(ex)
-                    log.error("Error fetching metadata: %s" % ex)
+                    #traceback.print_exc(ex)
+                    log.warn("problem fetching '%s' (will retry): %s" % (thread.url, ex))
                     if info is not None:
                         info['Exception'] = ex
                     if thread.tries < self.retry_limit:
                         next_jobs.append((thread.url, thread.verify, thread.id, thread.tries + 1))
                     else:
-                        log.error("Retry limit exceeded for %s" % thread.url)
+                        #traceback.print_exc(ex)
+                        log.error("retry limit exceeded for %s (last error was: %s)" % (thread.url, ex))
                 finally:
                     nfinished += 1
                     if info is not None:
@@ -403,7 +432,7 @@ and verified.
             with open(xrd, "w") as fd:
                 fd.write(template("trust.xrd").render(links=resolved))
 
-    def parse_metadata(self, fn, key=None, base_url=None, fail_on_error=False):
+    def parse_metadata(self, fn, key=None, base_url=None, fail_on_error=False, filter_invalid=True):
         """Parse a piece of XML and split it up into EntityDescriptor elements. Each such element
         is stored in the MDRepository instance.
 
@@ -414,12 +443,22 @@ and verified.
         try:
             t = etree.parse(fn, base_url=base_url, parser=etree.XMLParser(resolve_entities=False))
             t.xinclude()
-            schema().assertValid(t)
+            if filter_invalid:
+                for e in t.findall('{%s}EntityDescriptor' % NS['md']):
+                    if not schema().validate(e):
+                        error = _e(schema().error_log, m=base_url)
+                        log.debug("removing '%s': schema validation failed (%s)" % (e.get('entityID'), error))
+                        e.getparent().remove(e)
+                        self.fire(type=EVENT_DROP_ENTITY, url=base_url, entityID=e.get('entityID'), error=error)
+            else:
+            # Having removed the invalid entities this should now never happen...
+                schema().assertValid(t)
         except DocumentInvalid, ex:
-            log.debug(_e(ex.error_log))
-            raise MetadataException("XML schema validation failed")
+            traceback.print_exc()
+            log.debug("schema validation failed on '%s': %s" % (base_url, _e(ex.error_log, m=base_url)))
+            raise MetadataException("schema validation failed")
         except Exception, ex:
-            log.debug(_e(schema().error_log))
+            #log.debug(_e(schema().error_log))
             log.error(ex)
             if fail_on_error:
                 raise ex
@@ -427,7 +466,10 @@ and verified.
         if key is not None:
             try:
                 log.debug("verifying signature using %s" % key)
-                xmlsec.verify(t, key)
+                refs = xmlsec.verified(t, key)
+                if len(refs) != 1:
+                    raise MetadataException("XML metadata contains %d signatures - exactly 1 is required" % len(refs))
+                t = refs[0] # prevent wrapping attacks
             except Exception, ex:
                 tb = traceback.format_exc()
                 print tb
@@ -471,6 +513,7 @@ is stored in the MDRepository object.
                     self._index_entity(e)
                     ne += 1
 
+        self.fire(type=EVENT_IMPORTED_METADATA, size=ne, url=url)
         return ne
 
     def entities(self, t=None):
