@@ -57,12 +57,12 @@ from cherrypy.lib import cptools
 from cherrypy.process.plugins import Monitor, SimplePlugin
 from cherrypy.lib import caching
 from simplejson import dumps
-from pyff.constants import ATTRS
+from pyff.constants import ATTRS, EVENT_REPOSITORY_LIVE
 from pyff.locks import ReadWriteLock
 from pyff.mdrepo import MDRepository
 from pyff.pipes import plumbing
-from pyff.tools import _staticdirs
-from pyff.utils import resource_string, template, xslt_transform, dumptree, duration2timedelta
+from pyff.tools import _staticdirs, request_vhost
+from pyff.utils import resource_string, template, xslt_transform, dumptree, duration2timedelta, debug_observer
 from pyff.logs import log, SysLogLibHandler
 import logging
 from pyff.stats import stats
@@ -76,6 +76,9 @@ __author__ = 'leifj'
 site_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "site")
 cherrypy.tools.staticdirs = HandlerTool(_staticdirs)
 
+import i18n
+_ = i18n.language.ugettext
+
 
 class MDUpdate(Monitor):
     def __init__(self, bus, frequency=600, server=None):
@@ -91,6 +94,9 @@ class MDUpdate(Monitor):
             if self.lock.acquire(blocking=0):
                 locked = True
                 md = self.server.new_repository()
+                for o in self.server.observers:
+                    md.subscribe(o)
+
                 for p in server.plumbings:
                     state = {'update': True, 'stats': {}}
                     p.process(md, state)
@@ -100,6 +106,7 @@ class MDUpdate(Monitor):
                 with server.lock.writelock:
                     log.debug("update produced new repository with %d entities" % md.index.size())
                     server.md = md
+                    md.fire(type=EVENT_REPOSITORY_LIVE, size=md.index.size())
                     stats['Repository Update Time'] = datetime.now()
                     stats['Repository Size'] = md.index.size()
             else:
@@ -167,7 +174,7 @@ class MDStats(StatsPage):
         body = tree.getroot().find("body")
         body.tag = 'div'
         hstr = etree.tostring(body, pretty_print=True, method="html")
-        return template("basic.html").render(content=hstr, http=cherrypy.request)
+        return template("basic.html").render(content=hstr, http=cherrypy.request, _=_)
 
 
 class WellKnown():
@@ -283,7 +290,7 @@ Disallow: /
 
     @cherrypy.expose
     def finger(self, domain="localhost"):
-        return template("finger.html").render(http=cherrypy.request, domain=domain)
+        return template("finger.html").render(http=cherrypy.request, domain=domain, _=_)
 
     @cherrypy.expose
     def about(self):
@@ -294,6 +301,7 @@ Disallow: /
         version = pkg_resources.require("pyFF")[0].version
         return template("about.html").render(version=version,
                                              cversion=cherrypy.__version__,
+                                             _=_,
                                              sysinfo=" ".join(os.uname()),
                                              http=cherrypy.request,
                                              cmdline=" ".join(sys.argv),
@@ -306,13 +314,13 @@ Disallow: /
         """The /reset page clears all local browser settings for the device. After visiting
         this page users of the discovery service will see a "new device" page.
         """
-        return template("reset.html").render(http=cherrypy.request)
+        return template("reset.html").render(http=cherrypy.request, _=_)
 
     @cherrypy.expose
     def settings(self):
         """The /settings page documents the (non) use of cookies.
         """
-        return template("settings.html").render(http=cherrypy.request)
+        return template("settings.html").render(http=cherrypy.request, _=_)
 
     @cherrypy.expose
     def search(self, paged=False, query=None, page=0, page_limit=10, entity_filter=None):
@@ -366,7 +374,14 @@ class MDServer():
     """The MDServer class is the business logic of pyFF. This class is isolated from the request-decoding logic
     of MDRoot and from the ancilliary classes like MDStats and WellKnown.
     """
-    def __init__(self, pipes=None, autoreload=False, frequency=600, aliases=ATTRS, cache_enabled=True, hosts_dir=None):
+    def __init__(self,
+                 pipes=None,
+                 autoreload=False,
+                 frequency=600,
+                 aliases=ATTRS,
+                 cache_enabled=True,
+                 hosts_dir=None,
+                 observers=[]):
         if pipes is None:
             pipes = []
         self.cache_enabled = cache_enabled
@@ -376,6 +391,7 @@ class MDServer():
         self.refresh = MDUpdate(cherrypy.engine, server=self, frequency=frequency)
         self.refresh.subscribe()
         self.aliases = aliases
+        self.observers = observers
 
         if autoreload:
             for f in pipes:
@@ -391,7 +407,7 @@ class MDServer():
 
     md = property(_get_md, _set_md)
 
-    def new_repository(self):
+    def new_repository(self, observers=[]):
         return MDRepository(metadata_cache_enabled=self.cache_enabled)
 
     class MediaAccept():
@@ -496,6 +512,7 @@ class MDServer():
                     raise HTTPError(400, "400 Bad Request - Missing 'return' parameter")
                 pdict['returnIDParam'] = kwargs.get('returnIDParam', 'entityID')
                 pdict['suggest'] = self._guess_idp(entityID)
+                pdict['_'] = _
                 cherrypy.response.headers['Content-Type'] = 'text/html'
                 return template("ds.html").render(**pdict)
             elif ext == 's':
@@ -521,18 +538,22 @@ class MDServer():
                     if pfx:
                         title = pfx
                     else:
-                        title = "Metadata By Attributes"
+                        title = _("Metadata By Attributes")
                     return template("index.html").render(http=cherrypy.request,
                                                          md=self.md,
                                                          alias=alias,
                                                          aliases=self.aliases,
-                                                         title=title)
+                                                         title=title,
+                                                         _=_)
                 else:
                     entities = self.md.lookup(q)
                     if not entities:
                         raise NotFound()
                     if len(entities) > 1:
-                        return template("metadata.html").render(http=cherrypy.request, md=self.md, entities=entities)
+                        return template("metadata.html").render(http=cherrypy.request,
+                                                                _=_,
+                                                                md=self.md,
+                                                                entities=entities)
                     else:
                         entity = entities[0]
                         t = html.fragment_fromstring(unicode(xslt_transform(entity, "entity2html.xsl")))
@@ -545,7 +566,7 @@ class MDServer():
                             else:
                                 p.text = c_txt  # re.sub(".",escape,c_txt)
                         xml = dumptree(t, xml_declaration=False).decode('utf-8')
-                        return template("basic.html").render(http=cherrypy.request, content=xml)
+                        return template("basic.html").render(http=cherrypy.request, content=xml, _=_)
             else:
                 for p in self.plumbings:
                     state = {'request': True,
@@ -553,6 +574,7 @@ class MDServer():
                              'accept': accept,
                              'url': cherrypy.url(relative=False),
                              'select': q,
+                             'path': path,
                              'stats': {}}
                     r = p.process(self.md, state=state)
                     if r is not None:
@@ -572,8 +594,9 @@ def main():
     try:
         opts, args = getopt.getopt(sys.argv[1:],
                                    'hP:p:H:CfaA:l:',
-                                   ['help', 'loglevel=', 'log=', 'access-log=', 'error-log=', 'port=', 'host=',
-                                    'no-caching', 'autoreload', 'frequency=', 'alias=', 'dir=', 'version', 'proxy'])
+                                   ['help', 'loglevel=', 'log=', 'access-log=', 'error-log=', 'email=',
+                                    'port=', 'host=', 'no-caching', 'autoreload', 'frequency=',
+                                    'alias=', 'dir=', 'version', 'proxy'])
     except getopt.error, msg:
         print msg
         print __doc__
@@ -593,6 +616,7 @@ def main():
     aliases = ATTRS
     base_dir = None
     proxy = False
+    email = None
 
     try:
         for o, a in opts:
@@ -626,6 +650,8 @@ def main():
                 autoreload = True
             elif o in '--frequency':
                 frequency = int(a)
+            elif o in '--email':
+                email = a
             elif o in ('-A', '--alias'):
                 (a, sep, uri) = a.lpartition(':')
                 if a and uri:
@@ -671,6 +697,7 @@ def main():
 
     def error_page(code, **kwargs):
         kwargs['http'] = cherrypy.request
+        kwargs.setdefault('title', "pyFF @ %s" % request_vhost(cherrypy.request))
         return template("%d.html" % code).render(**kwargs)
 
     static_dirs = []
@@ -682,7 +709,17 @@ def main():
             static_dirs.append(os.path.join(hosts_dir, "%VHOST%"))
     static_dirs.append(site_dir)
 
-    server = MDServer(pipes=args, autoreload=autoreload, frequency=frequency, aliases=aliases, cache_enabled=caching)
+    observers = []
+
+    if loglevel == logging.DEBUG:
+        observers.append(debug_observer)
+
+    server = MDServer(pipes=args,
+                      autoreload=autoreload,
+                      frequency=frequency,
+                      aliases=aliases,
+                      cache_enabled=caching,
+                      observers=observers)
     pfx = ["/entities", "/metadata"] + ["/" + x for x in server.aliases.keys()]
     cfg = {
         'global': {
@@ -697,10 +734,10 @@ def main():
             'tools.caching.delay': 3600,  # this is how long we keep static stuff
             'tools.cpstats.on': True,
             'tools.proxy.on': proxy,
-            'error_page.404': lambda **kwargs: error_page(404, **kwargs),
-            'error_page.503': lambda **kwargs: error_page(503, **kwargs),
-            'error_page.500': lambda **kwargs: error_page(500, **kwargs),
-            'error_page.400': lambda **kwargs: error_page(400, **kwargs)
+            'error_page.404': lambda **kwargs: error_page(404, _=_, **kwargs),
+            'error_page.503': lambda **kwargs: error_page(503, _=_, **kwargs),
+            'error_page.500': lambda **kwargs: error_page(500, _=_, **kwargs),
+            'error_page.400': lambda **kwargs: error_page(400, _=_, **kwargs)
         },
         '/': {
             'tools.caching.delay': delay,
@@ -754,4 +791,4 @@ def main():
 
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    main()
