@@ -13,7 +13,6 @@ import os
 import re
 from pyff.logs import log
 import hashlib
-from eventlet.green import urllib2
 import xmlsec
 import base64
 from datetime import datetime
@@ -144,7 +143,10 @@ active document. To avoid this do a select before your fork, thus:
 def _any(lst, d):
     for x in lst:
         if x in d:
-            return d[x]
+            if type(d) == dict:
+                return d[x]
+            else:
+                return True
     return False
 
 
@@ -317,16 +319,6 @@ Publish the working document in XML form.
         req.md[resource_name] = req.t
     return req.t
 
-
-@deprecated
-def _fetch(md, url, verify):
-    log.debug("open %s" % url)
-    try:
-        return url, urllib2.urlopen(url).read(), verify, None, datetime.now()
-    except Exception, ex:
-        return url, None, None, ex, datetime.now()
-
-
 @deprecated
 def remote(req, *opts):
     """Deprecated. Calls :py:mod:`pyff.pipes.builtins.load`.
@@ -341,6 +333,11 @@ def local(req, *opts):
     return load(req, opts)
 
 
+@deprecated
+def _fetch(req, *opts):
+    return load(req,*opts)
+
+
 def load(req, *opts):
     """
 General-purpose resource fetcher.
@@ -843,8 +840,8 @@ Prepares the working document for publication/rendering.
 :param opts: Options (not used)
 :return: returns the working document with @Name, @cacheDuration and @validUntil set
 
-Set Name, cacheDuration and validUntil on the toplevel EntitiesDescriptor element of the working document. Unless
-explicit provided the @Name is set from the request URI if the pipeline is executed in the pyFF server. The
+Set Name, ID, cacheDuration and validUntil on the toplevel EntitiesDescriptor element of the working document. Unless
+explicit provided the @Name is set from the request URI if the pipeline is executed in the pyFF server. The @ID is set to a string representing the current date/time and will be prefixed with the string provided, which defaults to '_'. The
 @cacheDuration element must be a valid xsd duration (eg PT5H for 5 hrs) and @validUntil can be either an absolute
 ISO 8601 time string or (more comonly) a relative time on the form
 
@@ -864,6 +861,7 @@ If operating on a single EntityDescriptor then @Name is ignored (cf :py:mod:`pyf
     - finalize:
         cacheDuration: PT8H
         validUntil: +10d
+        ID: pyff
     """
     if req.t is None:
         raise PipeException("Your plumbing is missing a select statement.")
@@ -880,8 +878,9 @@ If operating on a single EntityDescriptor then @Name is ignored (cf :py:mod:`pyf
         if name is not None and len(name):
             e.set('Name', name)
 
+    IDprefix = req.args.get('ID', '_')
     if not e.get('ID'):
-        e.set('ID', datetime.now().strftime("pyff%Y%m%dT%H%M%S%Z"))
+        e.set('ID', datetime.now().strftime(IDprefix + "%Y%m%dT%H%M%S%Z"))
 
     validUntil = req.args.get('validUntil', e.get('validUntil', None))
     if validUntil is not None and len(validUntil) > 0:
