@@ -24,8 +24,19 @@ class PyffException(Exception):
     pass
 
 
-def _e(error_log):
-    return "\n".join(filter(lambda x: ":WARNING:" not in x, ["%s" % e for e in error_log]))
+def _e(error_log, m=None):
+    def _f(x):
+        if ":WARNING:" in x:
+            return False
+        if m is not None and not m in x:
+            return False
+        return True
+
+    return "\n".join(filter(_f, ["%s" % e for e in error_log]))
+
+
+def debug_observer(e):
+    log.error(repr(e))
 
 
 def resource_string(name, pfx=None):
@@ -45,6 +56,7 @@ This includes certain XSLT and XSD files.
 :param pfx: An optional prefix to use in searching for name
 
     """
+    name = os.path.expanduser(name)
     if os.path.exists(name):
         with open(name) as fd:
             return fd.read()
@@ -173,6 +185,7 @@ def safe_write(fn, data):
     """
     tmpn = None
     try:
+        fn = os.path.expanduser(fn)
         dirname, basename = os.path.split(fn)
         with tempfile.NamedTemporaryFile('w', delete=False, prefix=".%s" % basename, dir=dirname) as tmp:
             tmp.write(data)
@@ -225,7 +238,7 @@ class URLFetch(threading.Thread):
 
     def time(self):
         if self.isAlive():
-            raise ValueError("I'm not done yet")
+            raise ValueError("caller attempted to obtain execution time while fetcher still active")
         return self.end_time - self.start_time
 
     def run(self):
@@ -237,17 +250,17 @@ class URLFetch(threading.Thread):
 
         self.start_time = clock()
         try:
-            cache = None
-            if self.enable_cache:
-                log.debug("fetching %s using cache" % self.url)
-                cache = ".cache"
-            else:
-                log.debug("fetching %s without using cache" % self.url)
+            cache = httplib2.FileCache(".cache")
+            if not self.enable_cache:
+                log.debug("removing '%s' from cache" % self.url)
+                cache.delete(self.url)
+
+            log.debug("fetching '%s'" % self.url)
 
             if self.url.startswith('file://'):
                 path = self.url[7:]
                 if not os.path.exists(path):
-                    raise IOError("File not found: %s" % path)
+                    raise IOError("file not found: %s" % path)
 
                 with open(path, 'r') as fd:
                     self.result = fd.read()
@@ -255,21 +268,21 @@ class URLFetch(threading.Thread):
                     self.date = datetime.now()
                     self.last_modified = datetime.fromtimestamp(os.stat(path).st_mtime)
             else:
-                h = httplib2.Http(cache=cache, timeout=20,
-                                  disable_ssl_certificate_validation=True) # yes this is correct!
+                h = httplib2.Http(cache=cache, timeout=60,
+                                  disable_ssl_certificate_validation=True)  # trust is done using signatures over here
                 resp, content = h.request(self.url)
                 self.resp = resp
                 self.last_modified = _parse_date(resp.get('last-modified', resp.get('date', None)))
                 self.date = _parse_date(resp['date'])
                 if resp.status != 200:
-                    log.error("got %d: %s from %s" % (resp.status, resp.reason, self.url))
                     raise IOError(resp.reason)
                 self.result = content
                 self.cached = resp.fromcache
 
-            log.debug("got %d bytes from %s" % (len(self.result), self.url))
+            log.debug("got %d bytes from '%s'" % (len(self.result), self.url))
         except Exception, ex:
-            traceback.print_exc()
+            #traceback.print_exc()
+            #log.warn("unable to fetch '%s': %s" % (self.url, ex))
             self.ex = ex
             self.result = None
         finally:
@@ -329,5 +342,5 @@ def xslt_transform(t, stylesheet, params={}):
 def total_seconds(dt):
     if hasattr(dt, "total_seconds"):
         return dt.total_seconds()
-    else: 
+    else:
         return (dt.microseconds + (dt.seconds + dt.days * 24 * 3600) * 10 ** 6) / 10 ** 6
